module rmo
use rmo1rho
use sep3d_struct_mod

implicit none
type(sep3d),private :: sep_data,sep_rmo
integer :: nx,nz
real :: rho_o,rho_d
integer :: rho_n
complex,allocatable :: cdata(:,:,:)

contains
subroutine rmo_init()
integer, external :: sreed
real,allocatable :: rdata(:,:,:)
integer :: padx,padz
integer :: angle_i,ix
call from_param('rho_o',rho_o,1.0)
call from_param('rho_d',rho_d,0.1)
call from_param('rho_n',rho_n,1)

call init_sep3d("in",sep_data,"INPUT")
allocate(rdata(sep_data%n(1),sep_data%n(2),sep_data%n(3)))
if ( 4*sep_data%n(1)*sep_data%n(2)*sep_data%n(3) /= sreed("in",rdata,4*sep_data%n(1)*sep_data%n(2)*sep_data%n(3)) ) then
  call seperr("wrong read data")
end if
call from_param('padx',padx,sep_data%n(3))
call from_param('padz',padz,sep_data%n(1))
nx=sep_data%n(3)+padx; nz=sep_data%n(1)+padz

allocate(cdata(nz,sep_data%n(2),nx))
cdata=0.0
do ix=1,sep_data%n(3)
   do angle_i=1,sep_data%n(2)
      cdata(1:sep_data%n(1),angle_i,ix)=cmplx(rdata(:,angle_i,ix),0.0)
   end do
end do 
deallocate(rdata)
call fft_2d(cdata,+1)

call init_sep3d(sep_rmo,"OUTPUT","FLOAT","REGULAR",ctag="out")
call axis_allocate(sep_rmo,4)
sep_rmo%o(1:3)=sep_data%o
sep_rmo%d(1:3)=sep_data%d
sep_rmo%n(1:3)=sep_data%n
sep_rmo%o(4)=rho_o
sep_rmo%d(4)=rho_d
sep_rmo%n(4)=rho_n
call sep3d_set_sep3d(sep_rmo)
call sep3d_write_description("out",sep_rmo)

end subroutine rmo_init

subroutine rmo_de()
deallocate (cdata)
end subroutine rmo_de

subroutine fft_2d(cdata,signn)
integer :: signn
complex :: cdata(:,:,:)

integer ix,angle_i,iz
do ix=1,nx
  do angle_i=1,sep_data%n(2)
    call FFT1D(cdata(:,angle_i,ix),nz,signn) 
  end do
end do
do angle_i=1,sep_data%n(2)
  do iz=1,nz
    call FFT1D(cdata(iz,angle_i,:),nx,signn)
  end do
end do
end subroutine fft_2d

subroutine rmo_run()
complex,allocatable::tmp_cdata(:,:,:)
integer :: rho_i
real :: rho
integer,external :: srite
allocate(tmp_cdata(nz,sep_data%n(2),nx))
call rmo1rho_init(nz,nx,sep_data)
do rho_i=1,rho_n
   tmp_cdata=cdata
   rho=rho_o+(rho_i-1)*rho_d
   call rmo1rho_run(tmp_cdata,rho)
   call fft_2d(tmp_cdata,-1)
   if ( 4*sep_data%n(1)*sep_data%n(2)*sep_data%n(3) /= srite("out",real(tmp_cdata(1:sep_data%n(1),:,1:sep_data%n(3))),4*sep_data%n(1)*sep_data%n(2)*sep_data%n(3)) ) then
     call seperr("wrong read data")
   end if
end do 

deallocate(tmp_cdata)
end subroutine rmo_run

end module rmo
