module rmo1rho
use sep3d_struct_mod
use sep
implicit none
real :: kz_o,kz_d,kx_o,kx_d,angle_o,angle_d
integer:: kz_n,kx_n,angle_n
logical,private :: flat
real, private :: z0
contains

subroutine rmo1rho_init(nz,nx,sep_data)
type(sep3d)::sep_data
integer :: nz,nx
real :: pi
call from_param('flat',flat,.false.)
call from_param('z0', z0, 500.0)
pi=3.1415926
kz_o=-pi/sep_data%d(1)
kx_o=-pi/sep_data%d(3)
kz_d=2.0*pi/(nz*sep_data%d(1))
kx_d=2.0*pi/(nx*sep_data%d(3))
kz_n=nz
kx_n=nx
angle_o=sep_data%o(2)
angle_d=sep_data%d(2)
angle_n=sep_data%n(2)
end subroutine rmo1rho_init

subroutine rmo1rho_run(data,rho)
complex:: data(:,:,:)
real :: rho
! data(1,2,3) 1:kz 2:angle 3:kx

integer:: kz_i,kx_i,angle_i
real :: kz,kx,angle,pi
real :: cosa,sina,d_n,d_x,d_z,phx,phz
complex :: shift

pi=3.1415926

do kx_i=1,kx_n
   kx=kx_o+(kx_i-1)*kx_d
   do angle_i=1,angle_n
      angle=angle_o+(angle_i-1)*angle_d
      angle=angle/180.0*pi
      !angle=pi/3.0
      do kz_i=1,kz_n
         kz=kz_o+(kz_i-1)*kz_d
         if (kz==0) then
           if (kx==0) then
              cosa=0.0
              sina=0.0
           else
              cosa=0.0
              sina=1.0
           end if 
         else
           if (flat) then 
             cosa=1.0
             sina=0.0
           else
             cosa=abs(kz)/sqrt(kz*kz+kx*kx)
             sina=abs(kx)/sqrt(kz*kz+kx*kx)
           end if
         end if
         if (flat) then
           d_n=(1-rho)*tan(angle)**2*z0
         else 
           d_n=(1-rho)/(1-rho*(1-cosa))*sin(angle)*sin(angle)/(0.7-sin(angle)*sin(angle))*z0
         end if
         d_x=sina*d_n
         d_z=cosa*d_n
         phx=kx*d_x
         phz=kz*d_z
         shift=cmplx(cos(phz),sin(phz))*cmplx(cos(phx),sin(phx))
         data(kz_i,angle_i,kx_i)=data(kz_i,angle_i,kx_i)*shift
      end do
   end do
end do

end subroutine rmo1rho_run
end module rmo1rho
