module rot
use sep3d_struct_mod

implicit none
integer :: nx,nz,pos_pad_nx,pos_pad_nz
real    :: okx,okz,dkx,dkz,dx,dz,pi
complex,allocatable :: tmp_panel(:,:),tmp_panel_out(:,:)


contains
subroutine rot_init(nz1,nx1,dz1,dx1,nz_pad,nx_pad)
integer :: nx1,nz1
real :: dx1,dz1
!type(sep3d)::rot_in
integer :: nz_pad,nx_pad
pi=3.1415926
!nz=rot_in%n(1)
!nx=rot_in%n(2)
!dz=rot_in%d(1)
!dx=rot_in%d(2)
nz=nz1
nx=nx1
dz=dz1
dx=dx1
okx=-pi/dx
okz=-pi/dz
pos_pad_nx=nx_pad+nx
pos_pad_nz=nz_pad+nz
dkx=2.0*pi/(pos_pad_nx*dx)
dkz=2.0*pi/(pos_pad_nz*dz)
allocate(tmp_panel(pos_pad_nz,pos_pad_nx),tmp_panel_out(pos_pad_nz,pos_pad_nx) )
end subroutine rot_init

subroutine rotate_fourier(panel,cent_z,cent_x,cos_rot,sin_rot)
real :: panel(:,:)
real :: cent_z,cent_x,cos_rot,sin_rot
real :: ri_kz_new
integer :: ix,iz,i_kx_new,i_kz_new
real :: kx,kz,kz_new,kx_new,ri_kz_new,w_kz_new,ri_kx_new,w_kx_new

tmp_panel=0.0
tmp_panel(1:nz,1:nx)=cmplx(panel,0.0)
do ix=1,nx
   call FFT1D(tmp_panel(:,ix),pos_pad_nz,1)
end do
do iz=1,pos_pad_nz
   call FFT1D(tmp_panel(iz,:),pos_pad_nx,1)
end do

do ix=1,pos_pad_nx
  kx=okx+(ix-1)*dkx
  do iz=1,pos_pad_nz
    kz=okz+(iz-1)*dkz
    tmp_panel(iz,ix)=tmp_panel(iz,ix)*cexp(cmplx(0,-kz*cent_z-kx*cent_x))
  end do
end do

tmp_panel_out=0.0
do ix=1,pos_pad_nx
  kx=okx+(ix-1)*dkx
  do iz=1,pos_pad_nz
    kz=okz+(iz-1)*dkz
    kz_new=kz*cos_rot+kx*sin_rot
    kx_new=-kz*sin_rot+kx*cos_rot
    ri_kz_new=(kz_new-okz)/dkz+1.0
    i_kz_new=int(ri_kz_new)
    w_kz_new=ri_kz_new-i_kz_new
    ri_kx_new=(kx_new-okx)/dkx+1.0
    i_kx_new=int(ri_kx_new)
    w_kx_new=ri_kx_new-i_kx_new
    if (i_kx_new >=1 .and. i_kx_new <=pos_pad_nx ) then
      if (i_kz_new >=1 .and. i_kz_new <=pos_pad_nx ) then
         tmp_panel_out(iz,ix)=tmp_panel_out(iz,ix)+&
                              (1-w_kx_new)*(1-w_kz_new)*tmp_panel(i_kz_new,i_kx_new)+ &
                              (1-w_kx_new)* w_kz_new*tmp_panel(i_kz_new+1,i_kx_new) + &
                               w_kx_new*(1-w_kz_new)*tmp_panel(i_kz_new,i_kx_new+1) + &
                               w_kx_new*w_kz_new*tmp_panel(i_kz_new+1,i_kx_new+1)
      
      end if
    end if
   
  end do
end do
do ix=1,pos_pad_nx
  kx=okx+(ix-1)*dkx
  do iz=1,pos_pad_nz
    kz=okz+(iz-1)*dkz
    tmp_panel_out(iz,ix)=tmp_panel_out(iz,ix)*cexp(cmplx(0,kz*cent_z+kx*cent_x))
  end do
end do

do ix=1,pos_pad_nx
   call FFT1D(tmp_panel_out(:,ix),pos_pad_nz,-1)
end do
do iz=1,pos_pad_nz
   call FFT1D(tmp_panel_out(iz,:),pos_pad_nx,-1)
end do

panel=real(tmp_panel_out(1:nz,1:nx))
end subroutine rotate_fourier

subroutine rot_release()
deallocate(tmp_panel,tmp_panel_out)
end subroutine rot_release

end module rot
